;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME:   E2PROM_E.asm
;;  Version: 1.1, Updated on 2003/08/19 at 17:57:43
;;  Generated by PSoC Designer ver 4.0 b865 : 27 August, 2003
;;
;;  DESCRIPTION: EEPROM emulation User Module implementation file
;;     for CY8C26/7xxx and related devices.
;;
;;     Routines in this file perform E2PROM_E User Module Read and 
;;     Write operations.
;;
;;     They provide a layer of abstraction on top of the E2PROMLIB.asm routines
;;     to allow instancing of the EEPROM devices.  Together the E2PROM and
;;     E2PROMLIB algorithms emulate a byte-wise EEPROM device on a block oriented
;;     flash architecture.
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;     arguments and observe the associated "Registers are volatile" policy.
;;     This means it is the caller's responsibility to preserve any values
;;     in the X and A registers that are still needed after the API 
;;     function returns. Even though these registers may be preserved now,
;;     there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress MicroSystems 2001-2003. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "E2PROM_E.inc"

;-------------------------------------------------------------------
;  Declare the functions global for both assembler and C compiler.
;
;  Note that there are two names for each API. First name is 
;  assembler reference. Name with underscore is name refence for
;  C compiler.  Calling function in C source code does not require 
;  the underscore.
;-------------------------------------------------------------------

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------

export   E2PROM_E_Start
export  _E2PROM_E_Start
export   E2PROM_E_Stop
export  _E2PROM_E_Stop
export   E2PROM_E_bE2Write
export  _E2PROM_E_bE2Write
export   E2PROM_E_E2Read
export  _E2PROM_E_E2Read

export   E2PROM_E_RESERVED

;-----------------------------------------------------------------
;  Allocate the E2PROM_E EEPROM device in Flash memory
;     This will allow the linker to perform memory collision 
;     checking and the EEPROM device will be displayed in the 
;     map file.  
;     Note that this memory region can also be accessed from "C".
;-----------------------------------------------------------------
AREA  E2PROM_E_AREA   (ROM,ABS)
   ORG   E2PROM_E_START_ADDR
_E2PROM_E_RESERVED::
 E2PROM_E_RESERVED::     blkb     40h


AREA  UserModules (ROM,REL)

;-----------------------------------------------------------------------------
;  FUNCTION NAME: E2PROM_E_Start
;
;  DESCRIPTION:
;     Start routine for the E2PROM_E user module.  This routine is
;     a place holder for conformance and consistency.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     None
;
;  RETURNS:
;     None
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
.SECTION
 E2PROM_E_Start:
_E2PROM_E_Start:
   ret

.ENDSECTION


;-----------------------------------------------------------------------------
;  FUNCTION NAME: E2PROM_E_Stop
;
;  DESCRIPTION:
;     Stop routine for the E2PROM_E user module.  This routine is
;     a place holder for conformance and consistency.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     None
;
;  RETURNS:
;     None
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
.SECTION
 E2PROM_E_Stop:
_E2PROM_E_Stop:
   ret

.ENDSECTION


;-----------------------------------------------------------------------------
;  FUNCTION NAME: E2PROM_E_E2Write
;
;  DESCRIPTION:
;     Writes the specified E2PROM data at the wAddr and wByteCount from RAM into 
;     Flash into the defined E2PROM.
;
;     On entry - NOTE two entry points!!!
;           X must point to calling arguments in the following offset from X:
;              wAddr:         5  2-bytes
;              pbData:        3  2-bytes
;              wByteCount:    1  2-bytes
;              cTemperature:  0  1-byte
;
;     Prototype in C is:
;
;        #pragma  fastcall E2Write
;        void E2PROM_E_bE2Write( WORD wAddr, BYTE * pbData, WORD wByteCount, 
;                                        CHAR cTermperature );
;
;           (!!!note that C places arguments Right-to-Left onto the stack!!!)
;
;     Easiest method to call from assembly is as follows:
;           push  X
;           mov   X, SP
;           push  <cTemperature>
;           push  <wByteCount>
;           push  <wByteCount+1>
;           push  <0>                  ; MSB of pbDataDest is always 0 
;           push  <pbDataDest>
;           push  <wAddr>
;           push  <wAddr+1>
;           call  E2PROM_E_bE2Write
;           add   SP, -E2_RD_ARG_STACK_FRAME_SIZE 
;           pop   X
;
;           where <> refers to any addressing mode or number of instructions to
;              place the referenced data on the stack frame.
;
;           Other method is to create a stack frame using the defined equates and 
;           load them into the stack frame using [X+E2_WR_ARG_*] offset addressing mode.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     wAddr:         WORD   - relative OFFSET in defined E2PROM to write data
;     pbData:        BYTE * - pointer to the RAM buffer of data to write
;     wByteCount:    WORD   - number of bytes to write into E2PROM
;     cTemperature:  CHAR   - temperature in degrees celsius
;
;  RETURNS:    E2PROM_E_NOERROR, E2PROM_E_FAILURE, or 
;              E2PROM_E_STACKOVERFLOW
;
;  SIDE EFFECTS:  If a partial block is to be saved to flash, then a 64 byte buffer
;                 is temporary allocated.
;
;     REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;------------------------------------------------------------------------------
.SECTION
;------------------
; Assembly Entry
;------------------
 E2PROM_E_bE2Write::
   mov   A, E2PROM_E_START_BLOCK
   ljmp  bE2Write

;------------------
; 'C' Entry
;------------------
_E2PROM_E_bE2Write::
   mov   A, E2PROM_E_START_BLOCK
   ljmp  _bE2Write

.ENDSECTION


;-----------------------------------------------------------------------------
;  FUNCTION NAME: E2PROM_E_E2Read
;
;  DESCRIPTION:
;     Reads the specified E2PROM data at offset=wAddr for wByteCount bytes  and
;     places the data read into the RAM buffer pbDataDest.
;
;     On entry - NOTE two entry points!!!
;           X must point to calling arguments in the following offset from X:
;              wByteCount  0  2-bytes
;              pbDataDest  2  2-bytes
;              wAddr:      4  2-bytes
;
;     Prototype in C is:
;
;        #pragma  fastcall E2Read
;        void E2PROM_E_E2Read( WORD wAddr, BYTE * pbDataDest, WORD wByteCount );
;
;           (!!!note that C places arguments Right-to-Left onto the stack!!!)
;
;     Easiest method to call from assembly is as follows:
;           push  X
;           mov   X, SP
;           push  <wByteCount>
;           push  <wByteCount+1>
;           push  <0>                  ; MSB of pbDataDest is always 0 
;           push  <pbDataDest>
;           push  <wAddr>
;           push  <wAddr+1>
;           call  E2PROM_E_E2Read
;           add   SP, -E2_RD_ARG_STACK_FRAME_SIZE 
;           pop   X
;
;           where <> refers to any addressing mode or number of instructions to
;              place the referenced data on the stack frame.
;
;           Other method is to create a stack frame using the defined equates and 
;           load them into the stack frame using [X+E2_RD_ARG_*] offset addressing mode.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     bFirstBlockId  BYTE   - first block ID of E2PROM device
;     wAddr:         WORD   - relative OFFSET in defined E2PROM to read data
;     pbDataDest:    BYTE * - pointer to the RAM buffer to place read data
;     wByteCount:    WORD   - number of bytes to read from E2PROM
;
;  RETURNS:       none
;
;  SIDE EFFECTS:  
;     REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
.SECTION
;------------------
; Assembly Entry
;------------------
 E2PROM_E_E2Read::
   mov   A, E2PROM_E_START_BLOCK
   ljmp  E2Read

;------------------
; 'C' Entry
;------------------
_E2PROM_E_E2Read::
   mov   A, E2PROM_E_START_BLOCK
   ljmp  _E2Read
   
.ENDSECTION

; End of File   
